"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../errors");
const matcher_1 = require("./matcher");
class NumberMatcher extends matcher_1.Matcher {
    toBeLessThan(upperLimit) {
        this._validateValues(upperLimit, "toBeLessThan", "upper limit");
        if (this.actualValue < upperLimit !== this.shouldMatch) {
            throw new errors_1.LessThanMatchError(this.actualValue, upperLimit, this.shouldMatch);
        }
    }
    toBeGreaterThan(lowerLimit) {
        this._validateValues(lowerLimit, "toBeGreaterThan", "lower limit");
        if (this.actualValue > lowerLimit !== this.shouldMatch) {
            throw new errors_1.GreaterThanMatchError(this.actualValue, lowerLimit, this.shouldMatch);
        }
    }
    _validateValues(limit, functionName, limitType) {
        if (limit === null || limit === undefined) {
            throw new TypeError(`${functionName} ${limitType} must not be null or undefined.`);
        }
        if (typeof this.actualValue !== "number") {
            throw new TypeError(`${functionName} can only check numbers.`);
        }
    }
}
exports.NumberMatcher = NumberMatcher;
//# sourceMappingURL=number-matcher.js.map